<?php

namespace Sale\Handlers\PaySystem;

use Bitrix\Main;
use Bitrix\Main\Application;
use Bitrix\Main\Config\Option;
use Bitrix\Main\Error;
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Request;
use Bitrix\Main\Web\Json;
use Bitrix\Sale\Order;
use Bitrix\Sale\Payment;
use Bitrix\Sale\PaySystem;
use Bitrix\Sale\PriceMaths;
use Tinkoff\CreditBroker\Client;
use Tinkoff\CreditBroker\Receipt;
use Tinkoff\CreditBroker\ReceiptItem;

\CModule::IncludeModule("tinkoff.creditbroker");

class TCBHandler extends PaySystem\ServiceHandler
{
	/**
	 * @param Payment $payment
	 * @param Request|null $request
	 * @return PaySystem\ServiceResult
	 */
	public function initiatePay(Payment $payment, Request $request = null)
	{
		$result           = new PaySystem\ServiceResult();
		$paymentShouldPay = number_format(PriceMaths::roundPrecision($payment->getSum()), 2, '.', '');
		$order            = $payment->getCollection()->getOrder();

		if ($paymentShouldPay < 3000) {
			$showTemplateResult = $this->showTemplate($payment, "template_sum_error");
			if ($showTemplateResult->isSuccess()) {
				$result->setTemplate($showTemplateResult->getTemplate());
			} else {
				$result->addErrors($showTemplateResult->getErrors());
			}

			return $result;
		}

		$link = $this->getCreatedLink($payment);
		if ($link) {
			if (method_exists($result, "setPaymentUrl")) {
				$result->setPaymentUrl($link['link']);
			}
			$link['button_text'] = Loc::getMessage('TINKOFF.CREDITBROKER_BUTTON_TEXT_CREDIT_CONTINUE');
			$this->setExtraParams($link);
			$showTemplateResult = $this->showTemplate($payment, "template");
			if ($showTemplateResult->isSuccess()) {
				$result->setTemplate($showTemplateResult->getTemplate());
			} else {
				$result->addErrors($showTemplateResult->getErrors());
			}

			return $result;
		}

		$promocodes = Option::get("tinkoff.creditbroker", 'promocodes');
		if ($promocodes) {
			$promocodes = Json::decode($promocodes);
			$promocodes = array_filter($promocodes, function ($value, $key) {
				return $this->getBusinessValue($payment, 'TCB_PROMOCODE_' . strtoupper($key)) === 'Y';
			}, ARRAY_FILTER_USE_BOTH);
		}
		if (empty($promocodes)) {
			$promocodes = ['default' => Loc::getMessage('TINKOFF.CREDITBROKER_BUTTON_TEXT_CREDIT')];
		}

		if ($request === null) {
			$request = Main\Context::getCurrent()->getRequest();
		}

		if ($request->get('create') != 'new' || !$request->get('promo')) {
			$orderNumber         = $this->getBusinessValue($payment, 'ORDER_NUMBER');
			$data['amount']      = $paymentShouldPay;
			$data['promo']       = $promocodes;
			$data['orderId']     = $orderNumber;
			$data['paymentPath'] = $this->getBusinessValue($payment, 'TCB_PAYMENT_PATH');
			$data['paymentId']   = "$orderNumber/" . $payment->getId();
			$data['button_text'] = Loc::getMessage('TINKOFF.CREDITBROKER_BUTTON_TEXT_PAY');
			$this->setExtraParams($data);
			$showTemplateResult = $this->showTemplate($payment, "template_promo");
			if ($showTemplateResult->isSuccess()) {
				$result->setTemplate($showTemplateResult->getTemplate());
			} else {
				$result->addErrors($showTemplateResult->getErrors());
			}

			return $result;
		}

		$data = [
			'shopId'          => $this->getBusinessValue($payment, 'TCB_SHOP_ID'),
			'showcaseId'      => $this->getBusinessValue($payment, 'TCB_SHOWCASE_ID'),
			'orderNumber'     => $this->getBusinessValue($payment, 'ORDER_NUMBER'),
			'sum'             => $paymentShouldPay,
			'items'           => $this->getOrderItems($payment),
			'promoCode'       => $request->get('promo') ?: 'default',
			'values'          =>
			[
				'contact' => [
					'fio'         => [
						'lastName'   => $this->getLastName($order),
						'firstName'  => $this->getFirstName($order),
						'middleName' => $this->getMiddleName($order),
					],
					'mobilePhone' => $this->getPhone($order),
					'email'       => $this->getEmail($order),
				],
			],
			'webhookURL'      => $this->getNotifyUrl($payment),
			'failURL'         => $this->getFailUrl($payment),
			'successURL'      => $this->getSuccessUrl($payment),
			'returnURL'       => $this->getReturnUrl($payment),
			'integrationType' => '1c_bitrix_tcb',
		];
		if ($this->getBusinessValue($payment, 'TCB_DEMO') == 'Y' &&
			$this->getBusinessValue($payment, 'TCB_DEMO_FLOW')) {
			$data['demoFlow'] = $this->getBusinessValue($payment, 'TCB_DEMO_FLOW');
		}

		$createResult = $this->createPayment($payment, $data);
		if (!$createResult->isSuccess()) {
			$result->addErrors($createResult->getErrors());
			return $result;
		}

		$paymentData = $createResult->getData();
		$this->saveTransaction($data, $paymentData, $payment);
		LocalRedirect($paymentData['link'], true);
		/*
		if (method_exists($result, "setPaymentUrl")) {
		$result->setPaymentUrl($paymentData['link']);
		}

		$paymentData['amount']      = $paymentShouldPay;

		$this->setExtraParams($paymentData);

		$showTemplateResult = $this->showTemplate($payment, "template");
		if ($showTemplateResult->isSuccess()) {
		$result->setTemplate($showTemplateResult->getTemplate());
		} else {
		$result->addErrors($showTemplateResult->getErrors());
		}*/

		return $result;
	}

	public function getCurrencyList()
	{
		return ['RUB'];
	}

	public static function isMyResponse(Request $request, $paySystemId)
	{
		return $request->get('BX_PAYSYSTEM_CODE') === $paySystemId;
	}

	public function getPaymentIdFromRequest(Request $request)
	{
		return $request->get('BX_PAYMENT_ID');
	}

	public function processRequest(Payment $payment, Request $request)
	{
		$result          = new PaySystem\ServiceResult();
		$orderInfoResult = $this->getOrderInfo($payment);
		if (!$orderInfoResult->isSuccess()) {
			$result->addErrors($orderInfoResult->getErrors());
			return $result;
		}
		$orderInfo = $orderInfoResult->getData();

		$this->updateTransaction($orderInfo, $payment);

		if ($orderInfo['status'] === 'signed') {
			$description = Loc::getMessage('TINKOFF.CREDITBROKER_ORDER_SIGNED');
			$fields      = array(
				"PS_STATUS_CODE"        => mb_substr($orderInfo['status'], 0, 5),
				"PS_STATUS_DESCRIPTION" => $description,
				"PS_SUM"                => $orderInfo['order_amount'],
				"PS_STATUS"             => 'Y',
				"PS_RESPONSE_DATE"      => new Main\Type\DateTime(),
			);

			$result->setOperationType(PaySystem\ServiceResult::MONEY_COMING);

			$result->setPsData($fields);
		}
		return $result;
	}

	public function sendResponse(PaySystem\ServiceResult $result, Request $request)
	{
		global $APPLICATION;
		if (!$result->isSuccess()) {
			$APPLICATION->RestartBuffer();
			header("HTTP/1.1 500 " . $result->getErrorMessages()[0]);
		}
		return '';
	}

	protected function getOrderItems(Payment $payment)
	{
		$shipmentCollection = $payment
			->getCollection()
			->getOrder()
			->getShipmentCollection();

		$items = [
			'system' => new Receipt(),
			'other'  => new Receipt(),
		];
		$deliveryItem = null;

		foreach ($shipmentCollection as $shipment) {
			$itemKey = 'other';
			if ($shipment->isSystem()) {
				$itemKey = 'system';
			}
			$shipmentItemCollection = $shipment->getShipmentItemCollection();

			foreach ($shipmentItemCollection as $shipmentItem) {
				$basketItem = $shipmentItem->getBasketItem();
				if ($basketItem->isBundleChild()) {
					continue;
				}

				if (!$basketItem->getFinalPrice()) {
					continue;
				}

				if ($shipmentItem->getQuantity() <= 0) {
					continue;
				}
				$itemPrice = $basketItem->getPrice();
				if (method_exists($basketItem, "getPriceWithVat")) {
					$itemPrice = $basketItem->getPriceWithVat();
				}
				$items[$itemKey]->addItem(new ReceiptItem(
					$basketItem->getField('NAME'),
					number_format(PriceMaths::roundPrecision($itemPrice), 2, '.', ''),
					$shipmentItem->getQuantity()
				));

			}
			if ($shipment->getPrice() != 0 && !$shipment->isSystem()) {
				$deliveryItem = new ReceiptItem(
					$shipment->getDeliveryName(),
					number_format(PriceMaths::roundPrecision($shipment->getPrice()), 2, '.', '')
				);
			}
		}

		if (empty($items['other'])) {
			$items = $items['system'];
		} else {
			$items = $items['other'];
		}

		if ($deliveryItem) {
			$items->addItem($deliveryItem);
		}

		return $items->toArray();
	}

	protected function getNotifyUrl(Payment $payment)
	{
		$request = \Bitrix\Main\Application::getInstance()->getContext()->getRequest();
		$params  = [
			'BX_PAYSYSTEM_CODE' => $payment->getPaymentSystemId(),
			'BX_PAYMENT_ID'     => $payment->getId(),
		];
		$url = 'https://' . $request->getHttpHost() . '/bitrix/tools/tcbroker_sale_ps_result.php?' . http_build_query($params);
		return $url;
	}

	protected function getFailUrl(Payment $payment)
	{
		$request = \Bitrix\Main\Application::getInstance()->getContext()->getRequest();
		$failUrl = $this->getBusinessValue($payment, 'TCB_FAIL_URL');
		if (!$failUrl) {
			$failUrl = 'https://' . $request->getHttpHost() . '/bitrix/tools/sale_ps_fail.php';
		}

		$failUrl = $this->replaceIdPlaceholders($payment, $failUrl);

		return $failUrl;
	}

	protected function getReturnUrl(Payment $payment)
	{
		$request = \Bitrix\Main\Application::getInstance()->getContext()->getRequest();
		$failUrl = $this->getBusinessValue($payment, 'TCB_RETURN_URL');
		if (!$failUrl) {
			$failUrl = 'https://' . $request->getHttpHost() . '/bitrix/tools/sale_ps_fail.php';
		}

		$failUrl = $this->replaceIdPlaceholders($payment, $failUrl);

		return $failUrl;
	}

	protected function getSuccessUrl(Payment $payment)
	{
		$request    = \Bitrix\Main\Application::getInstance()->getContext()->getRequest();
		$successUrl = $this->getBusinessValue($payment, 'TCB_SUCCESS_URL');
		if (!$successUrl) {
			$successUrl = 'https://' . $request->getHttpHost() . '/bitrix/tools/sale_ps_success.php';
		}

		$successUrl = $this->replaceIdPlaceholders($payment, $successUrl);

		return $successUrl;
	}

	protected function replaceIdPlaceholders($payment, $link)
	{
		$replace = [
			'#ID#'           => $payment->getId(),
			'#ORDER_ID#'     => $payment->getCollection()->getOrder()->getId(),
			'#ORDER_NUMBER#' => urlencode(urlencode($this->getBusinessValue($payment, 'ORDER_NUMBER'))),
		];
		$link = str_replace(array_keys($replace), array_values($replace), $link);
		return $link;
	}

	protected function createPayment(Payment $payment, $data)
	{
		$result = new PaySystem\ServiceResult();

		$paymentCreated = $this->getCreatedLink($payment);
		if ($paymentCreated) {
			$result->setData($paymentCreated);
			return $result;
		}

		$api = $this->initApi($payment);
		try {
			$data     = Main\Web\Json::encode($data, JSON_UNESCAPED_UNICODE);
			$response = $api->create($data);
			$result->setData($response);
		} catch (\Exception $e) {
			$result->addError(new Error($e->getMessage()));
		}
		return $result;
	}

	protected function getCreatedLink(Payment $payment)
	{
		$connection = Application::getConnection();
		$sqlHelper  = $connection->getSqlHelper();
		$id         = $payment->getId();

		$row = $connection
			->query("
						SELECT * FROM tinkoffbank_credit where payment_id='" . $sqlHelper->forSql($id) . "'")
			->fetch();
		return $row;
	}

	protected function getOrderInfo(Payment $payment)
	{
		$result = new PaySystem\ServiceResult();

		$api = $this->initApi($payment);

		try {
			$response = $api->info($this->getBusinessValue($payment, 'ORDER_NUMBER'));
			$result->setData($response);
		} catch (\Exception $e) {
			$result->addError(new Error('Order info error: ' . $e->getMessage()));
		}
		return $result;
	}

	protected function commitPayment(Payment $payment)
	{
		$result = new PaySystem\ServiceResult();

		$api = $this->initApi($payment);

		$paymentShouldPay = number_format(PriceMaths::roundPrecision($payment->getSum()), 2, '.', '');
		$data             = [
			'amount'         => $paymentShouldPay,
			'items'          => $this->getOrderItems($payment),
			'prepaid_amount' => 0,
		];

		try {
			$data     = Main\Web\Json::encode($data, JSON_UNESCAPED_UNICODE);
			$response = $api->commit($this->getBusinessValue($payment, 'TCB_ORDER_NUMBER'), $data);
			$result->setData($response);
		} catch (\Exception $e) {
			$result->addError(new Error('Commit error: ' . $e->getMessage()));
		}
		return $result;
	}

	protected function getPhone(Order $order)
	{
		$phone = '';
		try {
			if ($propUserPhone = $order->getPropertyCollection()->getPhone()) {
				$phone = $propUserPhone->getValue();
			}
		} catch (\Exception $e) {
			return '';
		}

		$phone = Client::clearPhone($phone);
		return $phone;
	}

	protected function getFirstName(Order $order)
	{
		$name = '';
		try {
			if ($prop = $order->getPropertyCollection()->getPayerName()) {
				$name = $prop->getValue();
			}
		} catch (\Exception $e) {
			return '';
		}
		$name     = explode(' ', $name);
		$position = 0;
		if (count($name) > 1) {
			$position = 1;
		}
		return $name[$position];
	}

	protected function getLastName(Order $order)
	{
		$name = '';
		try {
			if ($prop = $order->getPropertyCollection()->getPayerName()) {
				$name = $prop->getValue();
			}
		} catch (\Exception $e) {
			return '';
		}
		$name = explode(' ', $name);
		if (count($name) < 2) {
			return '';
		}
		return $name[0];
	}

	protected function getMiddleName(Order $order)
	{
		$name = '';
		try {
			if ($prop = $order->getPropertyCollection()->getPayerName()) {
				$name = $prop->getValue();
			}
		} catch (\Exception $e) {
			return '';
		}
		$name = explode(' ', $name);
		if (count($name) < 3) {
			return '';
		}
		return $name[2];
	}

	protected function getEmail(Order $order)
	{
		$email = '';
		try {
			if ($prop = $order->getPropertyCollection()->getUserEmail()) {
				$email = $prop->getValue();
			}
		} catch (\Exception $e) {
			return '';
		}
		return $email;
	}

	protected function saveTransaction(array $data, array $paymentData, Payment $payment)
	{
		$connection = Main\Application::getConnection();
		$sqlHelper  = $connection->getSqlHelper();

		$data = [
			'id'           => $paymentData['id'],
			'order_number' => $data['orderNumber'],
			'link'         => $paymentData['link'],
			'order_id'     => $payment->getCollection()->getOrder()->getId(),
			'payment_id'   => $payment->getId(),
			'status'       => 'filling',
			'items'        => Main\Web\Json::encode($data['items'], JSON_UNESCAPED_UNICODE),
			'date'         => new Main\Type\DateTime(),
			'amount'       => $data['sum'],
		];

		$tableName = 'tinkoffbank_credit';

		$insert = $sqlHelper->prepareInsert($tableName, $data);

		$sql =
		"REPLACE INTO " . $sqlHelper->quote($tableName) . "(" . $insert[0] . ") " .
			"VALUES (" . $insert[1] . ")";

		$connection->queryExecute($sql);
	}

	protected function updateTransaction(array $orderInfo, Payment $payment)
	{
		$connection = Main\Application::getConnection();
		$sqlHelper  = $connection->getSqlHelper();
		$extra      = Main\Web\Json::encode($orderInfo, JSON_UNESCAPED_UNICODE);
		$connection->query("UPDATE tinkoffbank_credit set status='" . $sqlHelper->forSql($orderInfo['status']) . "',
			extra='" . $sqlHelper->forSql($extra) . "' where payment_id=" . $payment->getId());
	}

	protected function initApi(Payment $payment)
	{
		$showId     = $this->getBusinessValue($payment, 'TCB_SHOP_ID');
		$showcaseId = $this->getBusinessValue($payment, 'TCB_SHOWCASE_ID');
		$password   = $this->getBusinessValue($payment, 'TCB_PASSWORD');

		$api = new Client($showId, $showcaseId, $password);

		if ($this->getBusinessValue($payment, 'TCB_DEMO') == 'Y') {
			$api->setDemoMode();
		}

		return $api;
	}

}
